package com.nokia.financeapi.config.web;

import com.fasterxml.jackson.databind.ObjectMapper;
import com.google.gson.Gson;
import com.google.gson.reflect.TypeToken;
import com.nokia.financeapi.common.R;
import com.nokia.financeapi.utils.AESUtil;
import lombok.extern.slf4j.Slf4j;
import org.slf4j.MDC;
import org.springframework.lang.Nullable;
import org.springframework.util.StopWatch;
import org.springframework.util.StreamUtils;
import org.springframework.util.StringUtils;
import org.springframework.web.servlet.HandlerInterceptor;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.nio.charset.Charset;
import java.time.LocalDateTime;
import java.util.Enumeration;
import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * 请求日志拦截器
 */
@Slf4j
public class RequestLogHandlerInterceptor implements HandlerInterceptor {
    /**
     * 计时器线程变量
     */
    private static final ThreadLocal<StopWatch> STOP_WATCH_THREAD_LOCAL = new ThreadLocal<>();

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        StopWatch stopWatch = new StopWatch();
        stopWatch.start();
        // 计时器放入线程变量
        STOP_WATCH_THREAD_LOCAL.set(stopWatch);
        // 日志添加跟踪id
        MDC.put("traceId", UUID.randomUUID().toString().replace("-", ""));
        LocalDateTime now = LocalDateTime.now();
        log.info("请求地址: {} {}", request.getRequestURL().toString(), request.getMethod());
        // 请求头参数
        Map<String, String> headers = new HashMap<>();
        Enumeration<String> headerNames = request.getHeaderNames();
        while (headerNames.hasMoreElements()) {
            String k	= headerNames.nextElement();
            String v = request.getHeader(k);
            headers.put(k, v);
        }
        log.info("请求头参数: {}", new ObjectMapper().writeValueAsString(headers));
        // 查询参数
        Map<String, String> parameters = new HashMap<>();
        Enumeration<String> parameterNames = request.getParameterNames();
        while (parameterNames.hasMoreElements()) {
            String k	= parameterNames.nextElement();
            String v = request.getParameter(k);
            parameters.put(k, v);
        }
        log.info("查询参数: {}", new ObjectMapper().writeValueAsString(parameters));
        String body = null;
        if (!MyDispatcherServlet.UN_WRAPPER.contains(request.getRequestURI())) {
            // 请求体参数
            body = StreamUtils.copyToString(request.getInputStream(), Charset.forName(request.getCharacterEncoding()));
            log.info("请求体参数: {}", StringUtils.trimAllWhitespace(body));
        }
        // todo: 记录日志
        String token = request.getHeader("token");
        log.info("token: {}", token);
        if (!StringUtils.hasText(token)) {
            return forbidden(response);
        }
        if ("test_hebei".equals(token)) {
            return true;
        }
        try {
            String decodeToken = AESUtil.decrypt(token);
            log.info("decodeToken: {}", decodeToken);
            Gson gson = new Gson();
            Map<String, String> map = gson.fromJson(decodeToken, new TypeToken<Map<String, String>>() {
            }.getType());
            if (!"FINANCE".equals(map.get("APP_ID")) || !StringUtils.hasText(map.get("LOGIN_ID"))) {
                return forbidden(response);
            }
            map.put("api", request.getRequestURI());
            map.put("body", body);
            map.put("time", now.toString());
            log.info("调用记录: {}", gson.toJson(map));
        } catch (Exception e) {
            log.warn("token解密失败: {}", token);
            return forbidden(response);
        }
        return true;
    }

    @Override
    public void afterCompletion(HttpServletRequest request, HttpServletResponse response, Object handler,
                                @Nullable Exception ex) throws Exception {
        if (!MyDispatcherServlet.UN_WRAPPER.contains(request.getRequestURI())) {
            MyHttpServletResponseWrapper wrapper = (MyHttpServletResponseWrapper) response;
            String responseString = new String(wrapper.toByteArray());
            log.info("响应 {}: {}", wrapper.getStatus(), responseString);
            // 返回结果打印前1000个字符
//            log.info("返回 {}: {}", wrapper.getStatus(),
//                    org.apache.commons.lang3.StringUtils.substring(responseString, 0, 1000));
        } else {
            log.info("响应 {}", response.getStatus());
        }
        StopWatch stopWatch = STOP_WATCH_THREAD_LOCAL.get();
        stopWatch.stop();
        log.info("耗时 {} ms", stopWatch.getTotalTimeMillis());
        STOP_WATCH_THREAD_LOCAL.remove();
    }

    private boolean forbidden(HttpServletResponse response) throws IOException {
        response.setCharacterEncoding("UTF-8");
        response.setContentType("application/json; charset=utf-8");
        byte[] bytes = new Gson().toJson(R.error().code(403).message("没有权限访问")).getBytes();
        response.getOutputStream().write(bytes);
        return false;
    }
}